/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.action.services;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Strings;
import org.apache.commons.beanutils.NestedNullException;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.opengroup.osdu.register.utils.TimeLimitedMatcher;
import org.opengroup.osdu.register.action.model.ActionUriTemplate;
import org.opengroup.osdu.register.action.model.ParsedAction;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.InvocationTargetException;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;
import java.util.stream.Collectors;

@Service
public class ParseJsonIntoAction {

    @Autowired
    private JaxRsDpsLog log;

    private final TimeLimitedMatcher timeLimitedMatcher = new TimeLimitedMatcher();

    private final UrlTemplateRegexParser urlTemplateRegexParser = new UrlTemplateRegexParser();

    public ParsedAction parse(String url, JsonNode node) {
        // parse uris for templates
        List<ActionUriTemplate> actionUriTemplates;
        try {
            actionUriTemplates = this.parseUriTemplates(url);
        } catch (IllegalArgumentException e) {
            return getResponse(url, Collections.singletonList(e.getMessage()));
        }

        if (actionUriTemplates == null || actionUriTemplates.isEmpty()) {
            return getResponse(url, new ArrayList<>());
        }

        // return if there are parsing error
        List<String> errors = actionUriTemplates.stream().filter(au -> !Strings.isNullOrEmpty(au.getErrors())).map(ActionUriTemplate::getErrors).collect(Collectors.toList());
        if (!errors.isEmpty()) {
            return getResponse(url, errors);
        }

        Map<String, String> parsedUrlParams = new HashMap<>();
        for (ActionUriTemplate au : actionUriTemplates) {
            // find or parse the attribute in testPayload
            Object value = this.getPropertyValue(node, au.getTemplateVariable());
            if (value == null) {
                errors.add(String.format("Unable to parse %s from 'testPayload'.", au.getTemplateVariable()));
                continue;
            }

            String stringValue = String.valueOf(value);
            if (Strings.isNullOrEmpty(au.getRegex())) {
                parsedUrlParams.put(au.getTemplateVariable(), stringValue);
            } else {
                // apply the regex if found
                try {
                    Pattern pattern = Pattern.compile(au.getRegex());
                    Matcher matcher = this.timeLimitedMatcher.matcher(pattern, stringValue);
                    if (matcher.find()) {
                        if (matcher.groupCount() == 0) {
                            parsedUrlParams.put(au.getTemplateVariable(), matcher.group(0));
                        } else {
                            parsedUrlParams.put(au.getTemplateVariable(), matcher.group(1));
                        }
                    } else {
                        errors.add(String.format("Regex: %s failed to extract a value from the test payload.", au.getRegex()));
                    }
                } catch (TimeLimitedMatcher.RegexTimeOutException e) {
                    throw new AppException(HttpStatus.SC_BAD_REQUEST, "Regex matching time out", e.getMessage());
                }
            }
        }

        if (parsedUrlParams.size() < actionUriTemplates.size()) {
            return getResponse(url, errors);
        }

        return getResponse(this.updateUrlWithData(url, parsedUrlParams), errors);
    }

    private List<ActionUriTemplate> parseUriTemplates(String url) throws IllegalArgumentException {
        Map<String, String> parsedUris = this.urlTemplateRegexParser.resolveTemplateRegex(url);
        if (parsedUris == null) {
            this.log.info(String.format("No template found on action.url: %s", url));
            return new ArrayList<>();
        }

        List<ActionUriTemplate> actionUriTemplates = new ArrayList<>();
        parsedUris.forEach((template, regex) -> {
            if (Strings.isNullOrEmpty(regex)) {
                actionUriTemplates.add(ActionUriTemplate.builder().templateVariable(template).build());
            } else {
                if (this.isValidRegex(regex)) {
                    actionUriTemplates.add(ActionUriTemplate.builder().templateVariable(template).regex(regex).build());
                } else {
                    actionUriTemplates.add(ActionUriTemplate.builder()
                            .templateVariable(template)
                            .errors(String.format("Invalid regex: %s found for template var: %s", regex, template)).build());
                }
            }
        });
        return actionUriTemplates;
    }

    @SuppressWarnings("unchecked")
    private Object getPropertyValue(JsonNode testPayload, String template) {
        int separatorIndex = template.indexOf(":");
        String propertyKey;
        if (separatorIndex == -1) {
            propertyKey = template.substring(1, template.length() - 1);
        } else {
            propertyKey = template.substring(1, separatorIndex);
        }
        try {
            ObjectMapper mapper = new ObjectMapper();
            if (testPayload == null) {
                return null;
            }
            Map<String, Object> result = mapper.convertValue(testPayload, Map.class);

            // try getting first level property using optimized collection
            Object propertyVal = result.get(propertyKey);
            if (propertyVal != null) return propertyVal;

            // use apache utils to get nested property
            return PropertyUtils.getProperty(result, propertyKey);
        } catch (NestedNullException ignored) {
            // property not found in record
        } catch (IllegalArgumentException | IllegalAccessException | InvocationTargetException | NoSuchMethodException ignored) {
            // Unable to parse propertyKey from testPayload
        }
        return null;
    }

    boolean isValidRegex(String userInputPattern) {
        try {
            Pattern.compile(userInputPattern);
        } catch (PatternSyntaxException exception) {
            return false;
        }
        return true;
    }

    private ParsedAction getResponse(String url, List<String> errors) {
        return ParsedAction.builder()
                .url(url)
                .errors(StringUtils.join(errors, " ")).build();
    }

    private String updateUrlWithData(String urlWithTemplate, Map<String, String> templateAndValue) {
        for (Map.Entry<String, String> toBeReplaced : templateAndValue.entrySet()) {
            String template = toBeReplaced.getKey();
            String value = toBeReplaced.getValue();
            urlWithTemplate = urlWithTemplate.replace(template, value);
        }
        return urlWithTemplate;
    }
}
