/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.ddms.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Strings;
import com.reprezen.kaizen.oasparser.OpenApi3Parser;
import com.reprezen.kaizen.oasparser.model3.OpenApi3;
import com.reprezen.kaizen.oasparser.model3.Operation;
import com.reprezen.kaizen.oasparser.model3.Path;
import io.swagger.annotations.ApiModelProperty;
import lombok.AccessLevel;
import lombok.Data;
import lombok.Getter;
import lombok.Setter;
import org.opengroup.osdu.register.utils.UriResolver;
import org.opengroup.osdu.register.ddms.model.validators.ValidOpenApi3Spec;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.io.File;
import java.net.URI;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

@ValidOpenApi3Spec
@Data
public class RegisteredInterface {
    @NotBlank
    @Pattern(regexp = "^[A-Za-z0-9 ]{2,50}")
    private String entityType;

    @NotNull
    @ApiModelProperty(dataType = "object")
    private Map<String, Object> schema;

    @JsonIgnore
    @Getter(AccessLevel.NONE)
    @Setter(AccessLevel.NONE)
    private transient OpenApi3 openApi3;

    @JsonIgnore
    private static final UriResolver uriResolver = new UriResolver();

    public OpenApi3 openApi3Spec() throws Exception {
        if (openApi3 == null) {
            String temp = String.format("%s/%s.json", System.getProperty("java.io.tmpdir"), UUID.randomUUID().toString());
            ObjectMapper mapper = new ObjectMapper();
            File file = new File(temp);
            mapper.writeValue(file, schema);
            OpenApi3Parser parser = new OpenApi3Parser();
            openApi3 = parser.parse(file);
        }
        return openApi3;
    }

    public String retrieveSingleEntityPathString() throws Exception {
        if (!openApi3Spec().hasPaths())
            return "";

        Map<String, Path> paths = openApi3Spec().getPaths();
        List<Path> filteredPaths = paths
                .values()
                .stream()
                .filter(p -> {
                    Operation o = p.getGet();
                    if (o != null)
                        return o.hasExtension(OpenApiExtensionNames.RETRIEVE_ENTITY);
                    else
                        return false;
                })
                .collect(Collectors.toList());

        if (filteredPaths.size() == 1)
            return filteredPaths.get(0).getPathString();
        else
            return "";
    }

    public String retrieveServerBaseUrl() throws Exception {
        if (openApi3Spec().getServers().size() == 1)
            return openApi3Spec().getServer(0).getUrl();
        else
            return "";
    }

    public URI retrieveSingleEntityURI(String localid) throws Exception {
        String serverBaseUrl = retrieveServerBaseUrl();
        String path = retrieveSingleEntityPathString();
        if (Strings.isNullOrEmpty(path) || Strings.isNullOrEmpty(serverBaseUrl)) {
            return null;
        }
        return this.uriResolver.getUri(serverBaseUrl, path, localid);
    }
}
