/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.action.model;

import org.opengroup.osdu.register.utils.Helpers;
import org.junit.Test;

public class CreateActionDtoTest {

    @Test
    public void should_throwNoValidationErrors_when_allPropertiesAreOk(){
        CreateActionDto sut = validDto();

        Helpers.TestValidation(sut);

    }
    @Test
    public void should_throwValidationError_when_idContainsInvalidChar(){
        CreateActionDto sut = validDto();

        sut.setId("abc%123");

        Helpers.TestValidation(sut, "must match \"^[A-Za-z0-9-]{2,50}\"");
    }
    @Test
    public void should_throwValidationError_when_nameContainsInvalidChar(){
        CreateActionDto sut = validDto();

        sut.setName("abc%123");

        Helpers.TestValidation(sut, "must match \"^[A-Za-z0-9- ]{2,50}\"");
    }
    @Test
    public void should_throwValidationError_when_descriptionContainsInvalidChar(){
        CreateActionDto sut = validDto();

        sut.setDescription("abc%123<.");

        Helpers.TestValidation(sut, "must match \"^[A-Za-z0-9. ]{0,255}\"");
    }
    @Test
    public void should_throwValidationError_when_imgIsNotAURL(){
        CreateActionDto sut = validDto();

        sut.setUrl("abc%123<.");

        Helpers.TestValidation(sut, "must be a valid URL");
    }
    @Test
    public void should_throwValidationError_when_urlIsNotAURL(){
        CreateActionDto sut = validDto();

        sut.setImg("abc%123<.");

        Helpers.TestValidation(sut, "must be a valid URL");
    }
    @Test
    public void should_throwValidationError_when_filterContainsEmptyProperty(){
        CreateActionDto sut = validDto();

        sut.getFilter().getVersion().clear();

        Helpers.TestValidation(sut, "No filter value found. Must provide a filter value. Use '*' if you want to match all values for a filter property.");
    }
    @Test
    public void should_throwValidationError_when_filterContainsWildcardAndValue(){
        CreateActionDto sut = validDto();

        sut.getFilter().getSource().add("regular");

        Helpers.TestValidation(sut, "Can't provide wildcard '*' and specific filter values. Wildcard means you match all values for that property and must be used alone.");
    }
    @Test
    public void should_throwValidationError_when_filterContainsEmptyStringInProperty(){
        CreateActionDto sut = validDto();

        sut.getFilter().getEntityType().clear();
        sut.getFilter().getEntityType().add("");

        Helpers.TestValidation(sut, "Found empty filter values. Must provide a filter value. Use '*' if you want to match all values for a filter property.");
    }
    @Test
    public void should_throwValidationError_when_filterContainsMoreTHan30ValuesInProperty(){
        CreateActionDto sut = validDto();

        sut.getFilter().getEntityType().clear();
        for(int i = 0; i < 40; i++){
            sut.getFilter().getEntityType().add("ash");
        }
        Helpers.TestValidation(sut, "Can't provide more than 30 filter values for any 1 filter property.");
    }

    private  CreateActionDto validDto(){
        CreateActionDto dto = new CreateActionDto();
        dto.setId("123");
        dto.setName("123");
        dto.setDescription("123");
        dto.setUrl("https://my.com");
        dto.setImg("https://my.com/1.png");
        dto.getFilter().getEntityType().add("*");
        dto.getFilter().getVersion().add("*");
        dto.getFilter().getSource().add("*");
        return dto;
    }
}
