/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.middleware;

import javassist.NotFoundException;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.context.request.WebRequest;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.powermock.api.mockito.PowerMockito.mock;

import java.io.IOException;

@RunWith(MockitoJUnitRunner.class)
public class GlobalExceptionMapperTest {
    @Mock
    private JaxRsDpsLog log;

    @InjectMocks
    private GlobalExceptionMapper sut;

    @Test
    public void should_useValuesInAppExceptionInResponse_When_AppExceptionIsHandledByGlobalExceptionMapper() {

        AppException exception = new AppException(409, "any reason", "any message");

        ResponseEntity<Object> response = sut.handleAppException(exception);
        assertEquals(409, response.getStatusCodeValue());
        assertEquals("AppError(code=409, reason=any reason, message=any message, errors=null, debuggingInfo=null, originalException=null)", response.getBody().toString());
    }

    @Test
    public void should_addLocationHeader_when_fromAppException() {

        AppException exception = new AppException(302, "any reason", "any message");

        ResponseEntity<Object> response = sut.handleAppException(exception);
        assertEquals(302, response.getStatusCodeValue());
    }

    @Test
    public void should_useGenericResponse_when_exceptionIsThrownDuringMapping() {

        AppException exception = new AppException(302, "any reason", "any message");

        ResponseEntity<Object> response = sut.handleAppException(exception);

        assertEquals(302, response.getStatusCodeValue());
    }

    @Test
    public void should_use404ValueInResponse_When_NotFoundExceptionIsHandledByGlobalExceptionMapper() {
        NotFoundException exception = new NotFoundException("any message");

        ResponseEntity<Object> response = sut.handleNotFoundException(exception);

        assertEquals(404, response.getStatusCodeValue());
    }

    @Test
    public void should_use405ValueInResponse_When_HttpRequestMethodNotSupportedExceptionIsHandledByGlobalExceptionMapper() {
        HttpHeaders httpHeaders = mock(HttpHeaders.class);
        WebRequest webRequest = mock(WebRequest.class);
        HttpRequestMethodNotSupportedException exception = new HttpRequestMethodNotSupportedException("any message");

        ResponseEntity<Object> response = sut.handleHttpRequestMethodNotSupported(exception, httpHeaders, HttpStatus.METHOD_NOT_ALLOWED, webRequest);

        assertEquals(405, response.getStatusCodeValue());
    }

    @Test
    public void should_useGenericValuesInResponse_When_ExceptionIsHandledByGlobalExceptionMapper() {
		//Default Handler method commented in GlobalExceptionMapper Class
    	/*
		 * Exception exception = new Exception("any message");
		 * 
		 * ResponseEntity<Object> response = sut.handleGeneralException(exception);
		 * 
		 * assertEquals(500, response.getStatusCodeValue());
		 */
    }

    @Test
    public void should_returnNullResponse_when_BrokenPipeIOExceptionIsCaptured() {
        IOException ioException = new IOException("Broken pipe");

        ResponseEntity response = this.sut.handleIOException(ioException);

        assertNull(response);
    }

    @Test
    public void should_returnServiceUnavailable_when_IOExceptionIsCaptured() {
        IOException ioException = new IOException("Not broken yet");

        ResponseEntity response = this.sut.handleIOException(ioException);

        assertEquals(503, response.getStatusCodeValue());
    }
}