/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.subscriber.services;

import org.opengroup.osdu.core.common.model.notification.Topic;
import org.opengroup.osdu.register.subscriber.persistence.TopicsRepository;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.ArrayList;
import java.util.List;

import static junit.framework.TestCase.assertTrue;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class AvailableTopicsTest {

    @Mock
    TopicsRepository topicRepo;
    @InjectMocks
    AvailableTopics sut;

    @Before
    public void setup() {
        List<Topic> topics = new ArrayList<>();

        Topic topic1 = new Topic();
        topic1.setName("records-changed");
        topic1.setDescription("This notification is sent whenever a new record or record version is created/deleted, and when a new schema is created in storage.");
        topic1.setState("ACTIVE");
        topic1.setExample("{ \"id\": \"123456\"}");
        topics.add(topic1);

        Topic topic2 = new Topic();
        topic2.setName("records-changed2");
        topic2.setDescription("This notification is sent whenever a new record or record version is created/deleted, and when a new schema is created in storage.");
        topic2.setState("ACTIVE");
        topic2.setExample("{ \"name\": \"123456\"}");
        topics.add(topic2);
        when(topicRepo.listMessages()).thenReturn(topics);
    }

    @Test
    public void should_returnFalse_when_givenEmptyString() {
        assertFalse(sut.exists(""));
    }

    @Test
    public void should_returnFalse_when_givenMessageThatDoesNotExist() {
        assertFalse(sut.exists("not-exists"));
    }

    @Test
    public void should_returnFalse_when_givenMessageThatDoesExistButMissesCase() {
        assertFalse(sut.exists("records-chanGED"));
    }

    @Test
    public void should_returnTrue_when_givenMessageThatDoesExist() {
        assertTrue(sut.exists("records-changed"));
    }

    @Test
    public void should_returnAllMessages_when_listingThemByName() {
        assertEquals(2, sut.listNames().size());
        assertEquals("records-changed", sut.listNames().get(0));
        assertEquals("records-changed2", sut.listNames().get(1));
    }

    @Test
    public void should_returnAllMessages_when_listingThem() {
        assertEquals(2, sut.list().size());
        assertEquals("records-changed", sut.list().get(0).getName());
        assertEquals("records-changed2", sut.list().get(1).getName());
    }
}
